
from xmmsvalue import Reference
from xmmsvalue import Universe
from xmmsvalue import Equals
from xmmsvalue import NotEqual
from xmmsvalue import Match
from xmmsvalue import Smaller
from xmmsvalue import SmallerEqual
from xmmsvalue import Greater
from xmmsvalue import GreaterEqual
from xmmsvalue import Token
from xmmsvalue import Order
from xmmsvalue import Mediaset
from xmmsvalue import Limit
from xmmsvalue import IDList
from xmmsvalue import Queue
from xmmsvalue import PShuffle
from xmmsvalue import Union
from xmmsvalue import Intersection
from xmmsvalue import Complement
from xmmsvalue import Has

from xmmsvalue import coll_parse



#Py3k compat
try:
	a = basestring
	del a
except NameError:
	basestring = (bytes, str)

class PropDict(dict):
	def __init__(self, srcs):
		dict.__init__(self)
		self._sources = srcs

	def set_source_preference(self, sources):
		"""
		Change list of source preference
		This method has been deprecated and should no longer be used.
		"""
		raise DeprecationWarning("This method has been deprecated and should no longer be used. Set the sources list using the 'sources' property.")
		self._set_sources(sources)

	def has_key(self, item):
		try:
			self.__getitem__(item)
			return True
		except KeyError:
			return False

	def __contains__(self, item):
		return self.has_key(item)

	def __getitem__(self, item):
		try:
			return dict.__getitem__(self, item)
		except KeyError:
			if isinstance(item, basestring):
				for src in self._sources:
					if src.endswith('*'):
						for k in self:
							if k[0].startswith(src[:-1]) and k[1] == item:
								return dict.__getitem__(self, k)
					try:
						t = dict.__getitem__(self, (src, item))
						return t
					except KeyError:
						pass
			raise

	def get(self, item, default=None):
		try:
			return self[item]
		except KeyError:
			return default

	def _get_sources(self):
		return self._sources
	def _set_sources(self, val):
		if isinstance(val, basestring):
			raise TypeError("Need a sequence of sources")
		for i in val:
			if not isinstance(i, basestring):
				raise TypeError("Sources need to be strings")
		self._sources = val
	sources = property(_get_sources, _set_sources)




import qt

class XMMSConnector(qt.QObject):
	def __init__(self, parent, xmms):
		qt.QObject.__init__(self, parent)
		fd = xmms.get_fd()
		self.xmms = xmms
		self.xmms.set_need_out_fun(self.chkWr)

		self.rsock = qt.QSocketNotifier(fd, qt.QSocketNotifier.Read, self)
		self.connect(self.rsock, qt.SIGNAL("activated(int)"), self.do_read)
		self.rsock.setEnabled(False)

		self.wsock = qt.QSocketNotifier(fd, qt.QSocketNotifier.Write, self)
		self.connect(self.wsock, qt.SIGNAL("activated(int)"), self.do_write)
		self.wsock.setEnabled(False)

	def chkWr(self, i):
		if self.xmms.want_ioout():
			self.toggle_write(True)
		else:
			self.toggle_write(False)

	def toggle_read(self, bool):
		self.rsock.setEnabled(bool)

	def toggle_write(self, bool):
		self.wsock.setEnabled(bool)

	def do_read(self, i):
		self.xmms.ioin()

	def do_write(self, i):
		self.xmms.ioout()




# encoding: utf-8

from xmmsapi import \
		method_arg, \
		method_varg, \
		service_method, \
		service_broadcast, \
		service_constant, \
		XmmsServiceNamespace, \
		client_method, \
		client_broadcast, \
		XmmsServiceClient



#!/usr/bin/env python
# encoding: utf-8

from xmmsclient import *
from xmmsclient.service import *

class SampleService(XmmsServiceNamespace):
	"""Cool service"""
	namespace_path = ('org', 'xmms2', 'example',)

	answer = service_constant(42)
	somestring = service_constant("Hello, world!")

	hello = service_broadcast(doc = "Send hello to the world")

	@service_method()
	def getTheAnswer(self):
		"""Answer to life the universe and everything"""
		return self.answer

	@service_method(
		positional = (
			method_arg('message', 'string', "Message to emit", default = "Hello, world!"),
			),
		)
	def sayHello(self, message = "Hello, world!"):
		"""Request hello broadcast to be emitted"""
		self.hello.emit(message)


	class nested(XmmsServiceNamespace):
		"""A namespace in the service"""
		@service_method(
			positional = (
				method_arg('arg1', 'integer', "Argument 1"),
				method_arg('arg2', 'string', "Argument 2"),
				method_varg(),
				),
			named = (
				method_varg(),
				),
			)
		def test(self, arg1, arg2, *args, **kargs):
			"""Some test method"""
			return "sample.answer=%d; arg1=%d; arg2=%s; *=%r; **=%r" % (
				self.parent.getTheAnswer(),
				arg1,
				arg2,
				args,
				kargs,
				)


if __name__ == '__main__':
	import sys
	xs = XmmsLoop("ServiceClientTest")
	xs.connect()
	my_service = SampleService(xs)
	my_service.register()
	sys.stderr.write("Service registered (local id: %d)\n" % xs.client_id)
	sys.stderr.write("Ctrl+C to exit...\n")
	try:
		xs.loop()
	except KeyboardInterrupt:
		pass
